@@ -1,9 +1,10 @@
 import logging
-from re import Pattern
 from datetime import datetime, timedelta
 
 from django.conf import settings
 
+from .redis import KeycloakSessionStorage, GenericSessionStorage
+
 logger = logging.getLogger(__name__)
 
 
@@ -18,6 +19,7 @@ def get_cookie_equivalency(name=None, all_names=False):
         'oidc_id_token': 'dn_oit',
         'oidc_states': 'dn_os',
         'oidc_login_next': 'dn_oln',
+        'keycloak_session_id': 'dn_ksi',
     }
 
     return EQUIVALENCY.get(name) if not all_names else EQUIVALENCY
@@ -50,7 +52,8 @@ def generate_oidc_cookies(session, response):
         oidc_access_token = session.get('oidc_access_token', None)
         oidc_id_token = session.get('oidc_id_token', None)
         oidc_states = session.get('oidc_states', {})
-        oidc_login_next = ''
+        keycloak_session_id = session.get('keycloak_session_id', {})
+        oidc_login_next = session.get('oidc_login_next', '')
 
         # We need these tree variables to login, if at least one is None or empty, then return
         if not all([oidc_id_token_expiration, oidc_access_token, oidc_id_token]):
@@ -67,6 +70,9 @@ def generate_oidc_cookies(session, response):
         response.set_cookie(get_cookie_equivalency('oidc_id_token'), oidc_id_token, **extra_data)
         response.set_cookie(get_cookie_equivalency('oidc_states'), oidc_states, **extra_data)
 
+        if keycloak_session_id:
+            response.set_cookie(get_cookie_equivalency('keycloak_session_id'), keycloak_session_id, **extra_data)
+
     return response
 
 
@@ -87,3 +93,19 @@ def delete_oidc_cookies(response):
     response.delete_cookie('sessionid')
 
     return response
+
+
+def delete_user_sessions(keycloak_session_id: str):
+    try:
+        keycloak_session = KeycloakSessionStorage(keycloak_session_id, ".")
+        django_sessions = keycloak_session.load()
+        django_sessions = django_sessions.split(',') if django_sessions else []
+
+        for session in django_sessions:
+            logger.debug("Deleting django session: %s", session)
+            django_session = GenericSessionStorage(f"{settings.SESSION_REDIS_PREFIX}:{session}")
+            django_session.delete()
+
+        keycloak_session.delete()
+    except:
+        logger.exception("Failed to delete sessions using keycloak session %s", keycloak_session_id)
